;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;         Copyright (C) 2022,  Panetonent                                             ;;;
;;;                                                                                     ;;;
;;;         This program is free software: you can redistribute it and/or modify        ;;;
;;;         it under the terms of the GNU General Public License as published by        ;;;
;;;         the Free Software Foundation, either version 3 of the License, or           ;;;
;;;         (at your option) any later version.                                         ;;;
;;;                                                                                     ;;;
;;;         This program is distributed in the hope that it will be useful,             ;;;
;;;         but WITHOUT ANY WARRANTY; without even the implied warranty of              ;;;
;;;         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ;;;
;;;         GNU General Public License for more details.                                ;;;
;;;                                                                                     ;;;
;;;         You should have received a copy of the GNU General Public License           ;;;
;;;         along with this program.  If not, see <http://www.gnu.org/licenses/>        ;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.include "cpctelera.h.s"
.include "game.h.s"
.include "sys/physics.h.s"
.include "man/entity.h.s"
.include "man/input.h.s"
.include "sys/score.h.s"
.include "sys/render.h.s"
.include "man/animations.h.s"
.include "sys/generator.h.s"
.include "sys/ai.h.s"
.include "man/levels.h.s"
.include "sys/colision.h.s"
.include "man/int.h.s"
.include "tools/math.h.s"
.include "main.h.s"
.include "man/menu.h.s"
;;.include "music/music.h.s"


;; Reservamos memoria
    dead_counter:
        .db #0x00

    try_again_1:
        .asciz "Press"

    try_again_2:
        .asciz "[SPACE]"


;MAN_GAME_INIT
    ;Initializa todos los componentes para empezar el bucle del juego
    ;Input:
    ;Destrois: probablemente todo
    man_game_init::
        ld l, #char_color_background_2         ;Color de letra
        ld h, #char_color_background         ;Color de fondo
        call cpct_setDrawCharM0_asm
        call sys_render_clean

        call man_entity_init
        call sys_render_init
        call man_level_init
        call sys_score_init
        call sys_colision_init
        call sys_physics_init
        call man_input_init

        call sys_generator_create_player
        ;ld de, #_level1_song
        ;call man_music_Init
        ld hl, #cpct_akp_musicPlay_asm
        ld (function_ir_handler), hl
        call man_sfx_init


        ld hl, #man_game_update
        ld (game_state), hl
        

        ret
;//////////////////

;MAN_GAME_UPDATE
    ;Bucle del juego que gestiona, orquesta y organiza todos los componentes del mismo
    ;Inputs: 
    ;Destrois: todo probablemente
    man_game_update::
        ;; if(INT_RENDER_CAP < render_cap) { logica del game
        ld a, (render_cap)
        or a
        jr z, game_update_logic ;; if(render_cap == 0) { hacemos la logica normal
        ld d, a
        ld a, #INT_RENDER_CAP
        cp d
        jr nc, end_game_update  ;; else { salgo }
        
        ;; INT_RENDER_CAP < render_cap
            ld a, #0
            ld (render_cap), a
        game_update_logic:
            call sys_render_update
            call man_input_update
            call sys_ai_update
            call man_animation_update
            call sys_colision_update

            call sys_physics_update
            call man_entity_update
            call man_level_update
            call man_int_init
            call cpct_waitVSYNC_asm
            call man_sfx_play
            ; call cpct_akp_musicPlay_asm


            ld a, (lifes)        ;; Comprobamos si las vidas llegan a 0 para acabar la partida
            or a

            jr nz, end_game_update
            ; if(lifes == 0) { preparamos el estado del juego de game over
                ld hl, #man_game_over_init
                ld (game_state), hl

        ;; /////
        ; else { 
        end_game_update:

        ret
;//////////////////


;MAN_GAME_OVER_INIT
    ;Inicializa el estado de game over
    man_game_over_init::
        ld l, #char_color_white         ;Color de letra
        ld h, #char_color_black         ;Color de fondo
        call cpct_setDrawCharM0_asm

        ld hl, #cpct_akp_stop_asm
        ld (function_ir_handler), hl
        call man_music_death

        call sys_render_cleanScreenBlack

        call man_entity_init
        call sys_generator_create_dead_player
        ;;Cambiamos la animacion a la animacion de muerte
        call tools_math_drawtriangle

        call man_level_reset

        ld a, #GAME_DEAD_COUNTER
        ld (dead_counter), a

        ld hl, #man_game_over_animation
        ld (game_state), hl

        ret
;//////////////////

;MAN_GAME_OVER_ANIMATION
    ;Aplica la animacion de game over y saca la pantalla de (Pulsa espacio para jugar)
    man_game_over_animation:
        call man_animation_update
        call man_entity_get_player
        call sys_render_update_only_one_entity
        call cpct_waitVSYNC_asm

        ld hl, #cpct_akp_musicPlay_asm
        ld (function_ir_handler), hl

        ;; if(dead_counter == 0) 
        ld a, (dead_counter)
        dec a
        ld (dead_counter), a
        or a
        jr nz, end_game_over_animation
        ;; { escribimos mensaje y pasamos al estado de escuchar teclado
            ld iy, #try_again_1
            ld c, #TRY_AGAIN_1_X
            ld b, #TRY_AGAIN_1_Y
            call drawString

            ld iy, #try_again_2
            ld c, #TRY_AGAIN_2_X
            ld b, #TRY_AGAIN_2_Y
            call drawString

            call change_screen_to_show_C000

            ld hl, #man_game_over
            ld (game_state), hl
        ; } ///
        end_game_over_animation:
        ret
;/////////////////

;MAN_GAME_OVER
    ;Espera respuesta del jugador para volver a jugar
    man_game_over:
        ;; Estaneamos el teclado para actuar en consecuencia    
        ld a, (_cpct_akp_songLoopTimes)           ;/ guardamos en a el número de loops que lleva la canción
        cp #1                                     ;|
        jr nz, not_end_loop                       ;|y si no es 1 (empieza en 0, con lo que cuando da una vuelta pasa a 1), continúa su ejecución normal
        ld hl, #cpct_akp_stop_asm                 ;|si es 1, para la música
        ld (function_ir_handler), hl              ;\

        not_end_loop:
        call cpct_scanKeyboard_asm

        ld hl, #Key_Space
        call cpct_isKeyPressed_asm
        jr z, end_game_over
        ;;if(isKeyPressed('space')){
         ;; //TODO SPACE PRESSED
         ld hl, #man_menu_main_init
         ld (game_state), hl


        end_game_over:
        ret
;/////////////////


