;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;         Copyright (C) 2022,  Panetonent                                             ;;;
;;;                                                                                     ;;;
;;;         This program is free software: you can redistribute it and/or modify        ;;;
;;;         it under the terms of the GNU General Public License as published by        ;;;
;;;         the Free Software Foundation, either version 3 of the License, or           ;;;
;;;         (at your option) any later version.                                         ;;;
;;;                                                                                     ;;;
;;;         This program is distributed in the hope that it will be useful,             ;;;
;;;         but WITHOUT ANY WARRANTY; without even the implied warranty of              ;;;
;;;         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ;;;
;;;         GNU General Public License for more details.                                ;;;
;;;                                                                                     ;;;
;;;         You should have received a copy of the GNU General Public License           ;;;
;;;         along with this program.  If not, see <http://www.gnu.org/licenses/>        ;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.include "cpctelera.h.s"
.include "colision.h.s"
.include "man/entity.h.s" ; INCLUIDA EN COLISION.H.S
.include "sys/score.h.s"
.include "tools/math.h.s"
.include "sys/render.h.s"
.include "man/animations.h.s"
.include "sys/ai.h.s"
.include "man/int.h.s"


;;Reservar memoria
    restore_hp:
        .db #0x00


;SYS_COLISION_INIT
    ; Inicializa las colisiones
    ; Inputs
    sys_colision_init::
        ld a, #COLISIONS_PERFECT_HIT_COMBO
        ld (restore_hp), a
        ret
;/////////////////

;SYS_COLISION_ATACK_RIGHT
    ; Calcula las colisiones con el ataque de la derecha
    ; Inputs:
    ; Outputs:      A <- colision (0: no colision, n(no 0): si colision)
    ; Destrois:
    sys_colision_atack_right:
        ld a, e                                 ;; /
        cp #COLISION_ATACK_RIGHT_UMBRAL_0       ;; | right_atack_x_0 <= entity_x_1 
        jr c, no_colision_r                    ;; \
        
        ld a, #COLISION_ATACK_RIGHT_UMBRAL_3    ;; /
        cp d                                    ;; | entity_x_0 <= right_atack_x_3
        jr c, no_colision_r                    ;; \
        ; if(right_atack_x_0 <= entity_x_1 && entity_x_0 <= right_atack_x_3) {
            ld a, #COLISION_ATACK_RIGHT_UMBRAL_2    ;; /
            cp d                                    ;; | entity_x_0 <= right_atack_x_2
            jr c, good_colision_r                  ;; \
            ; if(entity_x_0 <= right_atack_x_2) {
                ld a, #COLISION_ATACK_RIGHT_UMBRAL_1    ;; /
                cp d                                    ;; | entity_x_0 <= right_atack_1
                jr c, great_colision_r                 ;; \
                ; if(entity_x_0 <= right_atack_1) {
                    ;; TODO perfect colision
                    call perfect_collision
                    ret ; jr end_orientation_if_r
                ; }
                ; else {
                great_colision_r:
                    ;; TODO great colision
                    call great_collision
                    ret ; jr end_orientation_if_r
                ; }
            ; }

            ; else {
            good_colision_r:
                ;; TODO good colision
                call good_collision
                ret ; jr end_orientation_if_r
            ; }
        ; }
        ; else {
            no_colision_r:
                ;; TODO no colision
                ld a, #0
                ret ; jr end_orientation_if_r
        ; }

        end_orientation_if_r:
        ret
;/////////////////


;SYS_COLISION_ATACK_LEFT
    ; Calcula las colisiones con el ataque de la izquierda
    ; Inputs:
    ; Outputs:      A <- colision (0: no colision, n(no 0): si colision)
    ; Destrois:
    sys_colision_atack_left:
        ld a, #COLISION_ATACK_LEFT_UMBRAL_3     ;; /
        cp d                                    ;; | enemy_x_0 <= left_atack_x_3
        jr c, no_colision_l                    ;; \

        ld a, e                                 ;; /
        cp #COLISION_ATACK_LEFT_UMBRAL_0        ;; | left_atack_x_0 <= enemy_x_1
        jr c, no_colision_l                    ;; \
        ; if(enemy_x_0 <= left_atack_x_3 && left_atack_x_0 <= enemy_x_1) {
            cp #COLISION_ATACK_LEFT_UMBRAL_1    ;; / RECUERDA: cp no modifica el valor de A, por lo tanto A sigue conteniendo E
            jr c, good_colision_l              ;; \ left_atack_x_1 <= enemy_x_1
            ; if(left_atack_x_1 <= enemy_x_1) {
                cp #COLISION_ATACK_LEFT_UMBRAL_2    ;; /
                jr c, great_colision_l             ;; \ left_atack_x_2 <= enemy_x_1
                ; if(left_atack_x_2 <= enemy_x_1) {
                    ;; TODO perfect colision
                    call perfect_collision
                    ret ; jr end_orientation_if_l
                ; }
                ; else {
                    great_colision_l:
                        ;;TODO great colision
                        call great_collision
                        ret ; jr end_orientation_if_l
                ; }
            ; }
            ; else {
                good_colision_l:
                    ;; TODO good colision
                    call good_collision
                    ret ; jr end_orientation_if_l
            ; }
        ; }
        ; else {
            no_colision_l:
                ;; TODO no colision
                ld a, #0
                ret ; jr end_orientation_if_l
        ; }

        end_orientation_if_l:
        ret
;/////////////////


;SYS_COLISION_UPDATE_ONE
    ; Actualiza las colisiones de una entidad
    ; Inputs:   IY <- Puntero a la entidad
    ; Destrois: 
    sys_colision_update_one:
        ;; Calculamos los umbrales de posicion del enemigo
        ld d, ent_x(iy)     ; enemy_x_0 = entidad['X']
        ld a, d             ; D = enemy_x_0
        add ent_width(iy)   ; enemy_x_1 = entidad['X'] + entidad['width']
        ld e, a             ; E = enemy_x_1

        push iy

        call man_entity_get_player      ;; / 
        ld a, ent_type(iy)              ;; | Comprobamos si el jugador esta atacando
        and #ENT_TYPE_ATACKING          ;; | 
        or a                            ;; \ 

        pop iy


        jr z, player_colision  ; player.isAtacking == FALSE --> nos saltamos las colisiones del ataque con la entidad
        ; if(player.isAtacking() == TRUE) {
            ;Cargamos los valores de la posicion de la entidad que va a colisionar en D y su otro extremo en E

            ;Comprobamos orientacion del jugador
            push iy
            
            call man_entity_get_player      ;; /
            ld a, ent_type(iy)              ;; | player['orientation']
            and #ENT_TYPE_ORIENTATION       ;; | z=1 left, z=0 right
            or a                            ;; \
            
            pop iy

            jr nz, orientation_left ;; player['orientation'] == LEFT    -->     orientation_left
            ; if(player['orientation'] == RIGHT) {
                orientation_right:
                call sys_colision_atack_right
                jr end_orientation_if
            ; }
            ; else (player['orientation'] == LEFT) {
                orientation_left:
                call sys_colision_atack_left
            ; }
            end_orientation_if:
                or a
                ; if(atackColision == FALSE) {
                    jr z, player_colision ;; Si el enemigo no colisiona con el ataque, comprobamos si lo hace con el player
                ; }
                ; else {
                    ;; Si el enemigo si ha colisionado con el ataque, la mandamos para morir y nos ahorramos las colisiones con el player y sumamos el score
                    ld d, #00
                    ld e, a
                    call sys_score_add
                    call man_entity_prepare_to_condemn
                    jr end_colisions
                ; }
        ; }
        ; else {
            player_colision:
            ;Colisiones de la entidad con el jugador
            ld a, ent_type(iy)          ;; /
            and #ENT_TYPE_ATACKING      ;; | Comprobamos que el enemigo este atacando 
            or a                        ;; \

            ; if(enemy['type'].contains(ATACKING) == FALSE)
                jr z, end_colisions   ;; Si la entidad no esta atacando, no colisiona con el player
            ; }
            ; else {
                ld a, #COLISION_PLAYER_X_1 ;; /
                cp d                       ;; | enemy_x_0 <= player_x_1
                jr c, end_colisions        ;; \

                ld a, e                    ;; /
                cp #COLISION_PLAYER_X_0    ;; | player_x_0 <= enemy_x_1
                jr c, end_colisions        ;; \
                ; if(enemy_x_0 <= player_x_1 && player_x_0 <= enemy_x_1) {
                 ;; TODO PLAYER COLISION
                 ld a, #COLISIONS_PERFECT_HIT_COMBO
                 ld (restore_hp), a

                 ld a, (lifes)
                 dec a
                 ld (lifes), a
                 
                 call sys_render_delete_one_life
                 call man_entity_condemn
                 push iy
                 call man_entity_get_player
                 ld ent_behavior_mayor(iy), #<sys_ia_playerblink
                 ld ent_behavior_menor(iy), #>sys_ia_playerblink
                 pop iy 
                ; }
            ; } 
        ; }
        end_colisions:
        ret
;/////////////////


;SYS_COLISION_UPDATE
    ; Gestiona las colisiones para todas las entidades
    ; Inputs:
    ; Outputs:
    ; Destrois:
    sys_colision_update::
        ld a, #ENT_TYPE_COLISION  ;;IMPORTANTE PARA EL FUTURO, TENEMOS QUE PONER QUE ESTE ATACANDO
        ld hl, #sys_colision_update_one
        call man_entity_forall_mask
        ret
;//////////////////

;PERFECT_COLLISION
    perfect_collision:
        ;; Actualizamos el contador del combo
        ld a, (restore_hp)
        dec a
        ld (restore_hp), a
        ;; if(restore_hp == 0) { heal
         or a
         jr nz, continue_prefect_collision
        ;; { heal
            ld a, #COLISIONS_PERFECT_HIT_COMBO  ;; Reseteamos el contador
            ld (restore_hp), a                  ;;
            
            ;; if(current_lifes != max_lifes)
             ld a, (max_lifes)
             ld b, a
             ld a, (lifes)
             cp b
             jr z, continue_prefect_collision
            ;; { heal
                inc a
                ld (lifes), a
                call sys_render_draw_one_life
            ;; }
            

        ;; }

        continue_prefect_collision:
         ld h, #0                ;/
         ld l, ent_ia_score(iy)   ;|
         ld b, h                 ;|Le sumamos el score a sí mismo respentando el formato decimal
         ld c, ent_ia_score(iy)   ;|       
         call tools_math_bcdadd  ;\
 
         ld b, h                 ;/
         ld c, ent_ia_score(iy)   ;|Triplicamos el score
         call tools_math_bcdadd  ;\
 
         ld a, l
         ld b, #3
         call man_entity_threshold_update
 
         push af
         ld a, #3
         call man_sfx_trigger
         pop af
    ret

;/////////////////

;GREAT_COLLISION
    great_collision:
        ;; Reseteamos el contador de combo
        ld a, #COLISIONS_PERFECT_HIT_COMBO
        ld (restore_hp), a

        ld h, #0                ;/
        ld l, ent_ia_score(iy)  ;|
        ld b, h                 ;|Le sumamos el score a sí mismo respentando el formato decimal
        ld c, ent_ia_score(iy)  ;|       
        call tools_math_bcdadd  ;\

        ld a, l
        ld b, #2
        call man_entity_threshold_update

        push af
        ld a, #2
        call man_sfx_trigger
        pop af
    ret

;/////////////////

;GOOD_COLLISION
    good_collision:
        ;; Reseteamos el contador de combo
        ld a, #COLISIONS_PERFECT_HIT_COMBO
        ld (restore_hp), a

        ld a, ent_ia_score(iy)
        ld b, #1
        call man_entity_threshold_update
        push af
        ld a, #1
        call man_sfx_trigger
        pop af
    ret

;/////////////////