;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;         Copyright (C) 2022,  Panetonent                                             ;;;
;;;                                                                                     ;;;
;;;         This program is free software: you can redistribute it and/or modify        ;;;
;;;         it under the terms of the GNU General Public License as published by        ;;;
;;;         the Free Software Foundation, either version 3 of the License, or           ;;;
;;;         (at your option) any later version.                                         ;;;
;;;                                                                                     ;;;
;;;         This program is distributed in the hope that it will be useful,             ;;;
;;;         but WITHOUT ANY WARRANTY; without even the implied warranty of              ;;;
;;;         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ;;;
;;;         GNU General Public License for more details.                                ;;;
;;;                                                                                     ;;;
;;;         You should have received a copy of the GNU General Public License           ;;;
;;;         along with this program.  If not, see <http://www.gnu.org/licenses/>        ;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.include "cpctelera.h.s"
.include "generator.h.s"
.include "render.h.s"
.include "man/animations.h.s"
.include "man/entity.h.s"
.include "man/levels.h.s"
.include "sprites/assets.h.s"
.include "man/menu.h.s"


;SYS_GENERATOR_CREATE_PLAYER
    ;Crea al personaje principal
    ;Outputs:       IY -> Posicion de memoria de la entidad
    ;Destrois:      AF, BC, DE, HL 
    sys_generator_create_player::
        ;; Jugador
        ld hl, #man_entity_player            
        call man_entity_create_with_template

        push iy

        ;; Ataque izquierdo
        ld hl, #man_entity_atack_left
        call man_entity_create_with_template


        ;; Ataque derecho
        ld hl, #man_entity_atack_right
        call man_entity_create_with_template

        ;; Umbral de ataque
        ld hl, #man_entity_threshold
        call man_entity_create_with_template

        pop iy
    ret
;/////////////////

;SYS_GENERATOR_CREATE_DEAD_PLAYER
    ;Crea al personaje muerto
    ;Outputs:       IY -> Posicion de memoria de la entidad
    ;Destrois:      AF, BC, DE, HL 
    sys_generator_create_dead_player::
        ;; Jugador
        ld hl, #man_entity_player            
        call man_entity_create_with_template


        ld ent_height(iy), #ENT_PLAYER_DEATH_HEIGHT
        ld ent_width(iy), #ENT_PLAYER_DEATH_WIDTH
        ld ent_y(iy), #ENT_PLAYER_DEATH_Y

        ld ent_animation_mayor(iy), #<animation_player_death
        ld ent_animation_menor(iy), #>animation_player_death

        ld ent_sprite_mayor(iy), #<_sp_player_death_1
        ld ent_sprite_menor(iy), #>_sp_player_death_1


    ret
;/////////////////

;;SYS_GENERATOR_CUSTOM_PLAYER
    ;Crea un personaje con ciertos parametros customizados y constantemente atacando
    ;Inputs:          B <- posicion x,   C <- posicion y, A <- orientacion
    ;Outputs:           IY -> direccion de memoria del jugador
    sys_generator_custom_player:
        push af
        push bc
        ;; Jugador
        ld hl, #man_entity_player            
        call man_entity_create_with_template


        pop bc              ;;
        ld ent_x(iy), b     ;; Sacamos de la pila los valores de BC para meterlos en el player
        ld ent_y(iy), c     ;; 

        pop af              ;; Sacamos AF para leer el valor de A
        push iy             ;; Metemos IY(direccion de memoria del player) en la pila para devolverlo al final
        push bc             ;; Metemos BC (pos x, pos y) en la pila porque los necesitamos mas adelante
        ;; if(orientation == 0) { salto a crear el ataque a la derecha
         or a
         jr z, custom_player_right
        ;; else { ataque a la izquierda
            ld b, #ENT_TYPE_ORIENTATION        ;;
            call man_entity_add_type           ;; Le añadimos al jugador el tipo atacando y el tipo orientacion

            ;; Cambiamos la animacion del player para que se reproduzca la animacion de ataque 
                ld ent_animation_mayor(iy), #<animation_player_leftattack_infinite
                ld ent_animation_menor(iy), #>animation_player_leftattack_infinite
            ;; ///

            ld hl, #man_entity_atack_left           ;; 
            call man_entity_create_with_template    ;; Creamos el ataque a la izquierda

            ld b, #(ENT_TYPE_RENDER | ENT_TYPE_ANIMATION)   ;; Le añadimos el tipo renderizable y animable al ataque
            call man_entity_add_type                        ;;

            ;; Seteamos posicions del ataque en relacion al player
                pop bc      ;; Posiciones del player
                
                ld a, b                 ;; A = pos_x_player
                sub #ENT_ATACK_WIDTH    ;; Calculamos la nueva posicion en X del ataque
                ld ent_x(iy), a         ;; pos_x_atack = pos_x_player - atack_width

                ld a, c                     ;; A = pos_y_player
                add #ENT_ATACK_RELATIVE_Y   ;; Calculamos la nueva posicion en Y del ataque
                ld ent_y(iy), a             ;; pos_y_atack = pos_y_atack - margin_atack
            ;; ///

            ;; Cambiamos la animacion del ataque para que se anime todo el rato atacando
                ld ent_animation_mayor(iy), #<animation_sword_leftattack_infinite
                ld ent_animation_menor(iy), #>animation_sword_leftattack_infinite
            ;; ///

            jr end_custom_player
        ;;}
        custom_player_right:
            ;; Cambiamos la animacion del player para que se reproduzca la animacion de ataque 
                ld ent_animation_mayor(iy), #<animation_player_rightattack_infinite
                ld ent_animation_menor(iy), #>animation_player_rightattack_infinite
            ;; ///

            ld hl, #man_entity_atack_right          ;; 
            call man_entity_create_with_template    ;; Creamos el ataque a la izquierda

            ld b, #(ENT_TYPE_RENDER | ENT_TYPE_ANIMATION)   ;; Le añadimos el tipo renderizable y animable al ataque
            call man_entity_add_type                        ;;

            ;; Seteamos posicions del ataque en relacion al player
                pop bc      ;; Posiciones del player
                
                ld a, b                 ;; A = pos_x_player
                add #ENT_PLAYER_WIDTH   ;; Calculamos la nueva posicion en X del ataque
                ld ent_x(iy), a         ;; pos_x_atack = pos_x_player + player_width

                ld a, c                     ;; A = pos_y_player
                add #ENT_ATACK_RELATIVE_Y   ;; Calculamos la nueva posicion en Y del ataque
                ld ent_y(iy), a             ;; pos_y_atack = pos_y_atack - margin_atack
            ;; ///

            ;; Cambiamos la animacion del ataque para que se anime todo el rato atacando
                ld ent_animation_mayor(iy), #<animation_sword_rightattack_infinite
                ld ent_animation_menor(iy), #>animation_sword_rightattack_infinite
            ;; ///

        jr end_custom_player




        end_custom_player:
            pop iy
        
        ret
;;///////////////////

;SYS_GENERATOR_SWITCHTORIGHT
    ;Actualiza la posicion de la entidad, asi como su mascara de tipos.
    ;Destrois: A, B
    sys_generator_switchToRight:
            ld ent_x(iy), #RIGHT_X_DEFAULT
            ld a, #ENT_TYPE_ORIENTATION        ;Le cambiamos la orientacion por defecto a la entidad
            xor #0xFF                          ;Negar una mascara
            ld b, a                            ;B = !ENT_TYPE_ORIENTATION 
            ld a, ent_type(iy)                 ;/           
            and b                              ;|Entidad.tipo.remove(ENT_TYPE_ORIENTATION)
            ld ent_type(iy), a                 ;\
    ret
;/////////////////

;SYS_GENERATOR_SEMIFUSA
    ;Genera un enemigo del tipo que indica el nombre de la funcion
    ;Inputs: A -> Direccion hacia la que mira el enemigo
    sys_generator_semifusa::
        ld hl, #man_entity_semifusa
        call man_entity_create_with_template 

        ;;;if(direccion == derecha){
            or a                    ;'A' vale 0 si la direccion es derecha
            ret nz
            ;;;@(direccion == derecha)
                ld ent_sprite_mayor(iy), #<_sp_semifusa_right               ;/
                ld ent_sprite_menor(iy), #>_sp_semifusa_right               ;\Entidad.sprite = right_sprite
                ld ent_animation_mayor(iy), #<animation_semifusa_rightrun   ;/
                ld ent_animation_menor(iy), #>animation_semifusa_rightrun   ;\Entidad.animation = right_animation
                call sys_generator_switchToRight
            ;;;
        ;;;}
    ret
;/////////////////

;SYS_GENERATOR_BLANCA
    ;Genera un enemigo del tipo que indica el nombre de la funcion
    ;Inputs: A -> Direccion hacia la que mira el enemigo
    sys_generator_blanca::
        ld hl, #man_entity_blanca
        call man_entity_create_with_template 

        ;;;if(direccion == derecha){
            or a                    ;'A' vale 0 si la direccion es derecha
            ret nz
            ;;;@(direccion == derecha)
                ld ent_sprite_mayor(iy), #<_sp_blanca_right               ;/
                ld ent_sprite_menor(iy), #>_sp_blanca_right               ;\Entidad.sprite = right_sprite
                ld ent_animation_mayor(iy), #<animation_blanca_rightrun   ;/
                ld ent_animation_menor(iy), #>animation_blanca_rightrun   ;\Entidad.animation = right_animation
                call sys_generator_switchToRight
            ;;;
        ;;;}
    ret
;/////////////////

;SYS_GENERATOR_NEGRA
    ;Genera un enemigo del tipo que indica el nombre de la funcion
    ;Inputs: A -> Direccion hacia la que mira el enemigo
    sys_generator_negra::
        ld hl, #man_entity_negra
        call man_entity_create_with_template 

        ;;;if(direccion == derecha){
            or a                    ;'A' vale 0 si la direccion es derecha
            ret nz
            ;;;@(direccion == derecha)
                ld ent_sprite_mayor(iy), #<_sp_negra_right               ;/
                ld ent_sprite_menor(iy), #>_sp_negra_right               ;\Entidad.sprite = right_sprite
                ld ent_animation_mayor(iy), #<animation_negra_rightrun   ;/
                ld ent_animation_menor(iy), #>animation_negra_rightrun   ;\Entidad.animation = right_animation
                call sys_generator_switchToRight
            ;;;
        ;;;}
    ret
;/////////////////

;SYS_GENERATOR_REDONDA
    ;Genera un enemigo del tipo que indica el nombre de la funcion
    ;Inputs: A -> Direccion hacia la que mira el enemigo
    sys_generator_redonda::
        ld hl, #man_entity_redonda
        call man_entity_create_with_template 

        ;;;if(direccion == derecha){
            or a                    ;'A' vale 0 si la direccion es derecha
            ret nz
            ;;;@(direccion == derecha)
                ld ent_sprite_mayor(iy), #<_sp_redonda_right               ;/
                ld ent_sprite_menor(iy), #>_sp_redonda_right               ;\Entidad.sprite = right_sprite
                ld ent_animation_mayor(iy), #<animation_redonda_right      ;/
                ld ent_animation_menor(iy), #>animation_redonda_right      ;\Entidad.animation = right_animation
                call sys_generator_switchToRight
            ;;;
        ;;;}
    ret
;/////////////////

;SYS_GENERATOR_SPIRIT
    ;Genera une fantasma en la posicion del enemigo
    ;Input:     B <- posicion x,   C <- posicion y
    ;Destrois:  A, HL
    ;Outpu:     IY <- puntero a entidad
    sys_generator_spirit::
        push bc

        ld hl, #man_entity_spirit                   ;; / Creamos el espiritu
        call man_entity_create_with_template        ;; \

        pop bc
        
        ld ent_x(iy), b             ;; / Le ponemos los valores que nos pasan en BC
        ld ent_y(iy), c             ;; \ Corresponden al X y al Y del fantasma

        ;Si el easterEgg esta activo cambiamos la animacion, el sprite y el tamaño
        ;;;if(easter == 1){
            ld a, (easter)
            or a
            ret z
            ;;;@(easter == 1)
                ld ent_width(iy), #SP_CARD_W
                ld ent_height(iy), #SP_CARD_H
                ld ent_animation_mayor(iy), #<animation_card
                ld ent_animation_menor(iy), #>animation_card
                ld ent_sprite_mayor(iy), #<_sp_card_top
                ld ent_sprite_menor(iy), #>_sp_card_top
            ;;;
        ;;;}
    ret
;/////////////////