;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;         Copyright (C) 2022,  Panetonent                                             ;;;
;;;                                                                                     ;;;
;;;         This program is free software: you can redistribute it and/or modify        ;;;
;;;         it under the terms of the GNU General Public License as published by        ;;;
;;;         the Free Software Foundation, either version 3 of the License, or           ;;;
;;;         (at your option) any later version.                                         ;;;
;;;                                                                                     ;;;
;;;         This program is distributed in the hope that it will be useful,             ;;;
;;;         but WITHOUT ANY WARRANTY; without even the implied warranty of              ;;;
;;;         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ;;;
;;;         GNU General Public License for more details.                                ;;;
;;;                                                                                     ;;;
;;;         You should have received a copy of the GNU General Public License           ;;;
;;;         along with this program.  If not, see <http://www.gnu.org/licenses/>        ;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.include "cpctelera.h.s"
.include "render.h.s"
.include "sprites/assets.h.s"
.include "man/entity.h.s"
.include "man/levels.h.s"
.include "score.h.s"

;; //////////////////////////////////
;; Memoria
    current_screen_to_draw:
        .dw 0x8000

    last_screen_to_draw:
        .dw 0xC000

    last_score:
        .dw 0x0001
    
    buffer_score:
        .db 1 

    last_entities_left_counter:
        .dw 0xFFFF

    buffer_entities_left_counter:
        .db 1   

    entities_left_label:
        .asciz "x"

;; //////////////////////////////////

;SYS_RENDER_MEM_INIT
    ;Inicializamos los parametros de la memoria de video
    sys_render_mem_init::
        ld c, #0
        call cpct_setVideoMode_asm

        ld hl, #_g_palette
        ld de, #16
        call cpct_setPalette_asm
        cpctm_setBorder_asm HW_BLACK
        ret
;/////////////////////////////////

;SYS_RENDER_INIT
    ;Inicializa el render
    ;Destrois:      AF, BC, DE, HL 
    sys_render_init::
        call drawBackground

        ;;Copiamos lo que hay en el primer buffer al segundo
        ld hl, #first_buffer
        ld de, #second_buffer
        ld bc, #0x4000
        ldir
    ret
;/////////////////

;SYS_RENDER_UPDATE_ONE_ENTITY
    ;Renderiza una entidad
    ;Inputs:    IY -> Posicion de memoria en la que empieza la entidad
    ;Destrois:  AF, HL, DE, BC    
    sys_render_update_one_entity::
        ; Comprobamos si el sprite y la posicion del enemigo han cambiado respecto al frame anterior.
        ; En caso contrario no renderizamos ese sprite.
        ;;;if(Entidad.penultimate_sprite == Entidad.sprite && Entidad.penultimate_memory_position == Entidad_memory_position){
           ;Condicion (Foldeada asi porque es muy larga)
            ;Primero comprobamos el sprite
            ld a, ent_last_sprite_mayor_2(iy)   ;/
            cp ent_sprite_mayor(iy)             ;| Comprobamos el primer byte del sprite
            jr nz, values_not_repited           ;\
            
            ld a, ent_last_sprite_menor_2(iy)   ;/
            cp ent_sprite_menor(iy)             ;| Comprobamos el segundo byte del sprite
            jr nz, values_not_repited           ;\

            ;Ahora comprobamos la posicion. Problema: la posicion actual viene dada por x, y y la penultima por una direccion de memoria
            ld hl, (current_screen_to_draw)     
            ex de, hl                           
            ld b, ent_y(iy)                     ;B = Entidad.y
            ld c, ent_x(iy)                     ;C = Entidad.x
            call cpct_getScreenPtr_asm          ;Devuelve en hl la posicion de memoria a dibujar

            ld a, ent_last_memory_position_mayor_2(iy)  ;/
            ld b, h                                     ;|
            cp b                                        ;|Comprobamos primer byte de la posicion
            jr nz, values_not_repited                   ;\

            ld a, ent_last_memory_position_menor_2(iy)  ;/
            ld b, l                                     ;|
            cp b                                        ;|Comprobamos primer byte de la posicion
            jr nz, values_not_repited                   ;\
           ;End condicion
            ;;;@(Entidad.last_sprite == Entidad.sprite && Entidad.last_memory_position == Entidad_memory_position)
                ;Aunque no se mueva ni cambie el sprite hay que actualizar los valores con la informacion de frames anteriores 
                call sys_render_updatepenultimatevalues
                call sys_render_updatelastvalues
                jr end_if_values_repited
            ;;;
            values_not_repited:
            ;;;@Else
                ;Si no se repiten todos los valores hay que volver a renderizar la entidad
                ld h, ent_render_function_menor(iy)
                ld l, ent_render_function_mayor(iy)
                jp (hl)
            ;;;
        end_if_values_repited:
        ;;;}
    ret
;/////////////////


;SYS_RENDER_UPDATE_CONTROLS
    ;Actualiza las entidades en la pantalla de controles
    sys_render_update_controls::
        ld a,   #ENT_TYPE_RENDER
        ld hl,  #sys_render_update_one_entity
        call man_entity_forall_mask


        call change_screen
        ret
;;////////////////


;SYS_RENDER_UPDATE
    ;Dibuja todos los sprites de las entidades en su posicion correspondiente
    ;Destrois:   AF, HL, DE, BC   
    sys_render_update::
        ld a,   #ENT_TYPE_RENDER
        ld hl,  #sys_render_update_one_entity
        call man_entity_forall_mask

        call drawEntitiesLeftCounter

        call drawScore
        
        call drawPuertas

        call change_screen
    ret
;/////////////////

;SYS_RENDER_CLEAN
    ;Borra ambos buffers de la memoria
    ;Destrois:   HL, AF, BC   
    sys_render_clean::
        ld de, #second_buffer
        xor a
        ld bc, #0x8000 
        call cpct_memset_asm
    ret
;/////////////////

;SYS_RENDER_DRAWSPRITE
    ;Dibuja un Sprite
    ;Inputs:        IY -> Posicion de memoria de la entidad 
    ;Output:        HL -> Posición de memoria en la que se ha pintado el sprite
    ;Destrois:      AF, BC, DE, HL 
    sys_render_drawSprite::
        ld hl, (current_screen_to_draw)
        ex de, hl
        ld b, ent_y(iy)             ;Posicion en y a pintar
        ld c, ent_x(iy)             ;Posicion en x a pintar
        call cpct_getScreenPtr_asm  ;Devuelve en hl la posicion de memoria a dibujar

        push hl                     ;Guardamos la posicion en la que se va a pintar

        ex de, hl                   ;DE = HL = Posicion de memoria de video a pintar
        ld l, ent_sprite_mayor(iy)  ; /
        ld h, ent_sprite_menor(iy)  ; \ HL = Entidad.Sprite
        ld b,  ent_width(iy)        ; B = Entidad.Width 
        ld c,  ent_height(iy)       ; C = Entidad.Height 
        call cpct_drawSpriteBlended_asm 

        pop hl                      ;Recuperamos la posicion en la que se ha pintado 
    ret
;/////////////////

;SYS_RENDER_FIRSTTIME
    sys_render_firstTime::
        ;;Paso 1: Pintar el sprite en la posicion en la que esta ahora 
        call sys_render_drawSprite
        
        ;;Paso 3: Actualizar los valores ultimos (que van a ser los que han sido los actuales este frame)
        call sys_render_updatelastvalues

        ld ent_render_function_mayor(iy), #<sys_render_secondTime
        ld ent_render_function_menor(iy), #>sys_render_secondTime
    ret
;/////////////////

;SYS_RENDER_SECONDTIME
    sys_render_secondTime:
        ;;Paso 1: Pintar el sprite en la posicion en la que esta ahora (En el segundo buffer)
        call sys_render_drawSprite

        ;;Paso 2: Actualizar los valores penultimos (que van a ser los que han sido ultimos este frame)
        call sys_render_updatepenultimatevalues

        ;;Paso 3: Actualizar los valores ultimos (que van a ser los que han sido los actuales este frame)
        call sys_render_updatelastvalues

        ld ent_render_function_mayor(iy), #<sys_render_nextTimes   ;/
        ld ent_render_function_menor(iy), #>sys_render_nextTimes   ;\Entidad.render_function = final_render_function
    ret
;/////////////////

;SYS_RENDER_NEXTTIMES
    sys_render_nextTimes:
        ;;Paso 1: Borrar el sprite en la posición de hace dos frames
        ld d, ent_last_memory_position_mayor_2(iy)      ;/
        ld e, ent_last_memory_position_menor_2(iy)      ;\ DE = Entidad.penultima_posicion 
        ld l, ent_last_sprite_mayor_2(iy)               ;/
        ld h, ent_last_sprite_menor_2(iy)               ;\ HL = Entidad.penultimo_sprite
        ld b, ent_width(iy)                             ; B = Entidad.width 
        ld c, ent_height(iy)                            ; C = Entidad.height 
        call cpct_drawSpriteBlended_asm

        ;;Paso 2: Pintar el sprite en la posicion en la que esta ahora
        call sys_render_drawSprite

        ;;Paso 3: Actualizar los valores penultimos (que van a ser los que han sido ultimos este frame)
        call sys_render_updatepenultimatevalues

        ;;Paso 4: Actualizar los valores ultimos (que van a ser los que han sido los actuales este frame)
        call sys_render_updatelastvalues
    ret
;/////////////////

;SYS_RENDER_DELETE
    ;Borra una entidad en ambos buffers
    ;Input: IY -> Puntero a entidad a borrar
    ;Destrois: 
    sys_render_delete::
        ;;Paso 1: Borrar el sprite en la posición de hace dos frames
        ld d, ent_last_memory_position_mayor_2(iy)      ;/
        ld e, ent_last_memory_position_menor_2(iy)      ;\ DE = Entidad.penultima_posicion 
        ld l, ent_last_sprite_mayor_2(iy)               ;/
        ld h, ent_last_sprite_menor_2(iy)               ;\ HL = Entidad.penultimo_sprite
        ld b, ent_width(iy)                             ; B = Entidad.width 
        ld c, ent_height(iy)                            ; C = Entidad.height 
        call cpct_drawSpriteBlended_asm

        ;;Paso 2: Borrar el sprite en la posición del frame pasado
        ld d, ent_last_memory_position_mayor(iy)      ;/
        ld e, ent_last_memory_position_menor(iy)      ;\ DE = Entidad.penultima_posicion 
        ld l, ent_last_sprite_mayor(iy)               ;/
        ld h, ent_last_sprite_menor(iy)               ;\ HL = Entidad.penultimo_sprite
        ld b, ent_width(iy)                           ; B = Entidad.width 
        ld c, ent_height(iy)                          ; C = Entidad.height 
        call cpct_drawSpriteBlended_asm
    ret
;/////////////////

;SYS_RENDER_UPDATELASTVALUES
    ;Actualiza los valores ultimos de la entidad asignandole los del frame actual
    ;Destrois: HL
    sys_render_updatelastvalues:
        ld ent_last_memory_position_mayor(iy), h    ;/
        ld ent_last_memory_position_menor(iy), l    ;\Entidad.last_position = Entidad.position    
        ld h, ent_sprite_mayor(iy)                  ;/
        ld l, ent_sprite_menor(iy)                  ;|
        ld ent_last_sprite_mayor(iy), h             ;|Entidad.last_sprite = Entidad.sprite
        ld ent_last_sprite_menor(iy), l             ;\
    ret
;/////////////////

;SYS_RENDER_UPDATEPENULTIMATEVALUES
    ;Actualiza los valores penultimos de la entidad asignandoles los del ultimo frame
    ;Destrois: A
    sys_render_updatepenultimatevalues:
        ld a, ent_last_memory_position_mayor(iy)    ;/
        ld ent_last_memory_position_mayor_2(iy), a  ;|
        ld a, ent_last_memory_position_menor(iy)    ;|Entidad.penultima_posicion = Entidad.ultima_posicion
        ld ent_last_memory_position_menor_2(iy), a  ;\
        ld a, ent_last_sprite_mayor(iy)             ;/
        ld ent_last_sprite_mayor_2(iy), a           ;|
        ld a, ent_last_sprite_menor(iy)             ;|Entidad.penultimo_sprite = Entidad.ultimo_sprite
        ld ent_last_sprite_menor_2(iy), a           ;\
    ret
;/////////////////

;DRAWBACKGROUND
    ;Dibuja el fondo a base de rectangulos y sprites
    ;Destrois:      AF, BC, DE, HL 
    drawBackground::
        ;Dibujamos el suelo
        call drawSuelo

        ;Dibujamos las columnas
        ld c, #COLUMN_1_X
        ld b, #COLUMN_1_Y
        call drawColumn

        ld c, #COLUMN_2_X
        ld b, #COLUMN_2_Y
        call drawColumn

        ld c, #COLUMN_3_X
        ld b, #COLUMN_3_Y
        call drawColumn

        ld c, #COLUMN_4_X
        ld b, #COLUMN_4_Y
        call drawColumn

        ;Dibujamos las velas sobre las columnas
        ld c, #VELA_1_X
        ld b, #VELA_1_Y
        call drawVela

        ld c, #VELA_2_X
        ld b, #VELA_2_Y
        call drawVela

        ld c, #VELA_3_X
        ld b, #VELA_3_Y
        call drawVela

        ld c, #VELA_4_X
        ld b, #VELA_4_Y
        call drawVela

        ;Dibujamos las ventanas
        ld c, #VENTANA_1_X
        ld b, #VENTANA_1_Y
        call drawVentana

        ld c, #VENTANA_2_X
        ld b, #VENTANA_2_Y
        call drawVentana

        ;Dibujamos el la caja del Boss
        call drawCajaBoss

        ;Dibujamos las puertas
        call drawPuertas

        ;Dibujamos el loguito de enemigos restantes y la 'X'
        call drawEnemiesLeftLabel
    ret
;/////////////////

;DRAWBYTE
    ;Dibuja el valor de un byte en pantalla
    ;Inputs: H -> Byte a pintar, C -> posX, B -> posY   
    ;Destrois: AF, HL, DE, BC, IX
    drawByte::
        ;Paso 1: Pintamos el valor grande de H
        push hl
        ld de, (current_screen_to_draw)
        call cpct_getScreenPtr_asm     
        ex de, hl                   ;DE = Posicion de memoria en la que pintar
        
        pop hl                      ;Recuperamos HL
        push de                     ;Vamos a necesitar más adelante la posicion en la pintamos el primer caracter asi que la guardamos
        push hl                     ;Guardamos HL

        ld a, h
        rra                         ;/
        rra                         ;|Rotamos 4 veces hacia la derecha y nos quedamos solo con los 4 bits inferiores.   
        rra                         ;|(Rotamos en vez de desplazar porque es más barato en memoria y en tiempo)
        rra                         ;|
        and #0b00001111             ;\
        add #ASCII_VALUE            ;Le sumamos la posicion del 0 en ascii
        ex de, hl                   ;HL = Posicion de memoria en la que pintar
        ld e, a
        call cpct_drawCharM0_asm

        ;Paso 2: Pintamos el valor perqueño de H
        pop hl          
        pop de         

        ld a, h
        and #0b00001111                 ;Esta vez no hace falta hacer rotaciones porque queremos el valor bajo
        add #ASCII_VALUE                ;Le sumamos la posicion del 0 en ascii
        ex de, hl                       ;HL = Posicion de memoria en la pintar
        ld de, #CHARACTER_BYTES_W 
        add hl, de
        ld e, a 
        call cpct_drawCharM0_asm
    ret
;/////////////////

;DRAWBYTE_C000
    ;Dibuja el valor de un byte en pantalla
    ;Inputs: H -> Byte a pintar, C -> posX, B -> posY   
    ;Destrois: AF, HL, DE, BC, IX
    drawByte_C000::
        ;Paso 1: Pintamos el valor grande de H
        push hl
        ld de, #0xC000
        call cpct_getScreenPtr_asm     
        ex de, hl                   ;DE = Posicion de memoria en la que pintar
        
        pop hl                      ;Recuperamos HL
        push de                     ;Vamos a necesitar más adelante la posicion en la pintamos el primer caracter asi que la guardamos
        push hl                     ;Guardamos HL

        ld a, h
        rra                         ;/
        rra                         ;|Rotamos 4 veces hacia la derecha y nos quedamos solo con los 4 bits inferiores.   
        rra                         ;|(Rotamos en vez de desplazar porque es más barato en memoria y en tiempo)
        rra                         ;|
        and #0b00001111             ;\
        add #ASCII_VALUE            ;Le sumamos la posicion del 0 en ascii
        ex de, hl                   ;HL = Posicion de memoria en la que pintar
        ld e, a
        call cpct_drawCharM0_asm

        ;Paso 2: Pintamos el valor perqueño de H
        pop hl          
        pop de         

        ld a, h
        and #0b00001111                 ;Esta vez no hace falta hacer rotaciones porque queremos el valor bajo
        add #ASCII_VALUE                ;Le sumamos la posicion del 0 en ascii
        ex de, hl                       ;HL = Posicion de memoria en la pintar
        ld de, #CHARACTER_BYTES_W 
        add hl, de
        ld e, a 
        call cpct_drawCharM0_asm
    ret
;/////////////////

;CHANGE_SCREEN
    ;Cambia la posicion de la pantalla usando codigo automodificable
    ;Destrois: HL, DE, BC
    change_screen:
        f_change_screen = . + 1         ;Contiene la direccion en la que esta el segundo byte de la instruccion jp 
        jp change_screen_to_show_8000
    ret
;/////////////////

;CHANGE_SCREEN_TO_8000
    ;Cambia la posicion de la pantalla a la 8000
    ;Destrois: HL, DE, BC
    change_screen_to_show_8000:
        ;;Ponemos para que se vea C000 mientras vamos a dibujar en 8000
        ld de, #0x0C20          
        call set_crtc

        ld hl, #0xC000
        ld (current_screen_to_draw), hl


        ld hl, #change_screen_to_show_C000
        ld (f_change_screen), hl
    ret
;/////////////////

;CHANGE_SCREEN_TO_C000
    ;Cambia la posicion de la pantalla a la C000
    ;Destrois: HL, DE, BC
    change_screen_to_show_C000::
        ;;Ponemos para que se vea 8000 mientras vamos a dibujar en C000
        ld de, #0x0C30          
        call set_crtc

        ld hl, #0x8000
        ld (current_screen_to_draw), hl

        ld hl, #change_screen_to_show_8000
        ld (f_change_screen), hl 
    ret
;/////////////////

;SET_CRTC
    ;Inputs: D -> Registro del crtc a cambiar, E -> Valor para ese registro
    ;Destrois: DE, BC
    set_crtc:
        ld bc, #0xBC00
        out (c), d
        inc b
        out (c), e
    ret
;/////////////////


;DRAWENTITIESLEFTCOUNTER
    ;Dibuja la puntuacion cada vez que se actualiza
    drawEntitiesLeftCounter::
        ;;;if(score == last_score){
            ld hl, (entities_left_counter)
            ld bc, (last_entities_left_counter)

            ld a, h                             
            cp b                                
            jr nz, different_entities_left_counter               
                
            ld a, l                             
            cp c            
            jr nz, different_entities_left_counter 

            ret                                     ;Si los scores son iguales salimos
            different_entities_left_counter:
            ;;;@(score != last_score)
                ;Pintamos el Contador de enemigos restantes
                ld hl, (entities_left_counter)  ;HL = Enemigos restantes (Hasta 9999)
                ld h, l                         ;Como nunca vamos a llegar a 100 enemigos en un nivel, solo pintamos el byte bajo
                ld b, #ENTITIES_LEFT_COUNTER_Y    
                ld c, #ENTITIES_LEFT_COUNTER_X
                call drawByte

                ;Problema: Para que el score se dibuje apropiadamente en el doble buffer 
                ;hay que dibujarlo en uno y el frame siguiente en el otro
                ;;;if(buffer_entities_left_counter == 0){
                    ld a, (buffer_entities_left_counter)
                    or a
                    jr nz, entities_left_counter_no_pintado_en_ambos_buffers 
                    ;;;@(buffer_score == 0)
                        ld hl, (entities_left_counter)
                        ld (last_entities_left_counter), hl
                        ld a, #1
                        ld (buffer_entities_left_counter), a
                        jr end_if_entities_left_counter_buffer
                    ;;;
                    entities_left_counter_no_pintado_en_ambos_buffers:
                    ;;;@(buffer_score != 0)
                        ld a, (buffer_entities_left_counter)
                        dec a
                        ld (buffer_entities_left_counter), a
                    ;;;
                end_if_entities_left_counter_buffer:
                ;;;}
            ;;;    
        ;;;}
        
    ret
;/////////////////

;DRAWSCORE
    ;Dibuja la puntuacion cada vez que se actualiza
    drawScore::
        ;;;if(score == last_score){
            ld hl, (score)
            ld bc, (last_score)

            ld a, h                             
            cp b                                
            jr nz, different_scores               
                
            ld a, l                             
            cp c            
            jr nz, different_scores 

            ret                                     ;Si los scores son iguales salimos
            different_scores:
            ;;;@(score != last_score)
                ;Pintamos el byte mayor del score
                ld hl, (score)
                ld b, #SCORE_Y    
                ld c, #SCORE_X
                call drawByte

                ;Pintamos el byte menor del score
                ld hl, (score)
                ld h, l
                ld b, #SCORE_Y    
                ld c, #(SCORE_X+2*CHARACTER_BYTES_W) ;Hay que dibujar en la misma posicion que antes, dos caracteres mas a la derecha
                call drawByte

                ;Problema: Para que el score se dibuje apropiadamente en el doble buffer 
                ;hay que dibujarlo en uno y el frame siguiente en el otro
                ;;;if(buffer_score == 0){
                    ld a, (buffer_score)
                    or a
                    jr nz, score_no_pintado_en_ambos_buffers 
                    ;;;@(buffer_score == 0)
                        ld hl, (score)
                        ld (last_score), hl
                        ld a, #1
                        ld (buffer_score), a
                        jr end_if_score_buffer
                    ;;;
                    score_no_pintado_en_ambos_buffers:
                    ;;;@(buffer_score != 0)
                        ld a, (buffer_score)
                        dec a
                        ld (buffer_score), a
                    ;;;
                end_if_score_buffer:
                ;;;}
            ;;;    
        ;;;}
        
    ret
;/////////////////

;SYS_RENDER_UPDATE_ONLY_ONE_ENTITY
    ;Logica de render y doble buffer pero para una sola entidad
    ;Inputs:  IY <- entidad a actualizar
    sys_render_update_only_one_entity::
        call sys_render_update_one_entity
        call change_screen
        ret
;/////////////////

;DRAWCOLUMN
    ;Dibuja una columna del fondo
    ;Inputs: C -> posX, B -> posY
    ;Destrois: AF, HL, BC, DE
    drawColumn:
        push bc                     ;Guardamos BC (la posicion en X e Y) para más adelante

        ;Obtenemos la posicion de la base de arriba de la columna
        ld de, #first_buffer
        call cpct_getScreenPtr_asm

        ;Dibujamos la base de arriba de la columna
        ex de, hl
        ld a, #background_color_2
        ld c, #COLUMN_BASE_W
        ld b, #COLUMN_BASE_H
        call cpct_drawSolidBox_asm
        
        pop bc                      ; Recuperamos BC

        ;Obtenemos la posicion de la semibase de arriba de la columna
        ld de, #first_buffer
        ld a, c                         ;/
        add a, #COLUMN_DISPLACEMENT     ;| Desplazamos la X uno a la derecha
        ld c, a                         ;\
        ld a, b                         ;/
        add a, #COLUMN_BASE_H           ;| Deplazamos la Y el alto de la base hacia abajo
        ld b, a                         ;\
        push bc                         ;Guardamos el nuevo valor de BC
        call cpct_getScreenPtr_asm

        ;Dibujamos la semibase de arriba de la columna
        ex de, hl
        ld a, #background_color_2
        ld c, #COLUMN_SEMIBASE_W
        ld b, #COLUMN_SEMIBASE_H
        call cpct_drawSolidBox_asm

        pop bc                      ; Recuperamos BC

        ;Obtenemos la posicion del centro de la columna
        ld de, #first_buffer
        ld a, c                                 ;/
        add a, #(COLUMN_DISPLACEMENT)           ;| Desplazamos la X a la derecha
        ld c, a                                 ;\
        ld a, b                                 ;/
        add a, #(COLUMN_SEMIBASE_H)             ;| Deplazamos la Y el alto de la base hacia abajo
        ld b, a                                 ;\
        push bc                                 ;Guardamos el nuevo valor de BC
        call cpct_getScreenPtr_asm

        ;Dibujamos el centro de la columna
        ex de, hl
        ld a, #background_color_2
        ld c, #COLUMN_CENTER_W
        ld b, #COLUMN_CENTER_H
        call cpct_drawSolidBox_asm

        pop bc                      ; Recuperamos BC
        push bc                                 ;Guardamos el nuevo valor de BC

        ;Obtenemos la posicion del brillo del centro de la columna
        ld de, #first_buffer
        ld a, c                                 ;/
        add a, #(COLUMN_BRIGHT_DISPLACEMENT)    ;| Desplazamos la X a la derecha
        ld c, a                                 ;\
        call cpct_getScreenPtr_asm

        ;Dibujamos el brillo del centro de la columna
        ex de, hl
        ld a, #COLUMN_BRIGHT_COLOR
        ld c, #COLUMN_BRIGHT_W
        ld b, #COLUMN_BRIGHT_H
        call cpct_drawSolidBox_asm

        pop bc                      ; Recuperamos BC

        ;Obtenemos la posicion de la semibase de arriba de la columna
        ld de, #first_buffer
        ld a, c                         ;/
        add a, #-COLUMN_DISPLACEMENT    ;| Desplazamos la X uno a la derecha
        ld c, a                         ;\
        ld a, b                         ;/
        add a, #COLUMN_CENTER_H         ;| Deplazamos la Y el alto de la base hacia abajo
        ld b, a                         ;\
        push bc                         ;Guardamos el nuevo valor de BC
        call cpct_getScreenPtr_asm

        ;Dibujamos la semibase de arriba de la columna
        ex de, hl
        ld a, #background_color_2
        ld c, #COLUMN_SEMIBASE_W
        ld b, #COLUMN_SEMIBASE_H
        call cpct_drawSolidBox_asm

        pop bc                      ; Recuperamos BC

        ;Obtenemos la posicion de la base de arriba de la columna
        ld de, #first_buffer
        ld a, c                         ;/
        add a, #-COLUMN_DISPLACEMENT    ;| Desplazamos la X uno a la derecha
        ld c, a                         ;\
        ld a, b                         ;/
        add a, #COLUMN_SEMIBASE_H       ;| Deplazamos la Y el alto de la base hacia abajo
        ld b, a                         ;\
        call cpct_getScreenPtr_asm

        ;Dibujamos la base de arriba de la columna
        ex de, hl
        ld a, #background_color_2
        ld c, #COLUMN_BASE_W
        ld b, #COLUMN_BASE_H
        call cpct_drawSolidBox_asm
    ret
;/////////////////

;DRAWVELA
    ;Dibuja una vela
    ;Inputs: C -> posX, B -> posY
    ;Destrois: AF, HL, BC, DE
    drawVela:
        ld de, #first_buffer
        call cpct_getScreenPtr_asm

        ld de, #_sp_vela
        ex de, hl
        ld c, #SP_VELA_W
        ld b, #SP_VELA_H
        call cpct_drawSprite_asm
    ret
;/////////////////

;DRAWVENTANA
    ;Dibuja una ventana
    ;Inputs: C -> posX, B -> posY
    ;Destrois: AF, HL, BC, DE
    drawVentana:
        push bc

        ;Obtenemos la posicion de la ventana
        ld de, #first_buffer
        call cpct_getScreenPtr_asm

        ;Dibujamos la ventana
        ld de, #_sp_ventana
        ex de, hl
        ld c, #SP_VENTANA_W
        ld b, #SP_VENTANA_H
        call cpct_drawSprite_asm

        pop bc
        push bc

        ;Obtenemos la posicion de la decoracion superior de la ventana
        ld de, #first_buffer
        ld a, b
        add #-SP_DECORACION_VENTANA_H
        ld b, a
        ld a, c
        add #DECT_VENTANA_X_DISPLACEMENTS
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos la decoracion superior de la ventana
        ld de, #_sp_decoracion_ventana
        ex de, hl
        ld c, #SP_DECORACION_VENTANA_W
        ld b, #SP_DECORACION_VENTANA_H
        call cpct_drawSprite_asm


        pop bc
        ;Obtenemos la posicion de la base de la ventana
        ld de, #first_buffer
        ld a, b
        add #SP_VENTANA_H
        ld b, a
        ld a, c
        add #VENTANA_BASE_DISPLACEMENTS
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos la base de la ventana
        ex de, hl
        ld a, #background_color_2
        ld c, #VENTANA_BASE_W
        ld b, #VENTANA_BASE_H
        call cpct_drawSolidBox_asm
    ret
;/////////////////

;DRAWSUELO
    ;Dibuja el suelo
    ;Destrois: AF, HL, BC, DE
    drawSuelo:
        ;Obtenemos la posicion de la primera mitad del suelo
        ld de, #first_buffer
        ld c, #SUELO_X
        ld b, #SUELO_Y
        push bc
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #background_color_2
        ld c, #(SUELO_W/2)
        ld b, #SUELO_H
        call cpct_drawSolidBox_asm

        pop bc
        ;Obtenemos la posicion de la segunda mitad del suelo
        ld de, #first_buffer
        ld a, c 
        add a, #(SUELO_W/2)
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos la segunda mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #background_color_2
        ld c, #(SUELO_W/2)
        ld b, #SUELO_H
        call cpct_drawSolidBox_asm


        ;Obtenemos la posicion de la primera mitad de la base del suelo
        ld de, #first_buffer
        ld c, #SUELO_X
        ld b, #SUELO_BASE_Y 
        push bc
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad de la base del suelo
        ex de, hl
        ld a, #background_color_2
        ld c, #(SUELO_W/2)
        ld b, #SUELO_BASE_H
        call cpct_drawSolidBox_asm

        pop bc
        ;Obtenemos la posicion de la segunda mitad de la base del suelo
        ld de, #first_buffer
        ld a, c 
        add a, #(SUELO_W/2)
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos la segunda mitad de la base del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #background_color_2
        ld c, #(SUELO_W/2)
        ld b, #SUELO_BASE_H
        call cpct_drawSolidBox_asm
    ret
;/////////////////

;SYS_RENDER_DRAW_ONE_LIFE
    ;Dibuja una vida
    sys_render_draw_one_life::
        ;; DIBUJAR UNA VIDA
            ;; Memoria de video
             ld de, #0xC000
             ld a, (current_life_x)
             ld c, a
             ld b, #SYS_SCORE_LIFE_Y
             call cpct_getScreenPtr_asm
 
             ld de, #_sp_vida
             ex de, hl
             ld c, #SYS_SCORE_LIFE_HEIGHT
             ld b, #SYS_SCORE_LIFE_WIDTH
             call cpct_drawSpriteBlended_asm
            ;;///

            ;; Doble buffer
             ld de, #0x8000
             ld a, (current_life_x)
             ld c, a
             ld b, #SYS_SCORE_LIFE_Y
             call cpct_getScreenPtr_asm
 
             ld de, #_sp_vida
             ex de, hl
             ld c, #SYS_SCORE_LIFE_HEIGHT
             ld b, #SYS_SCORE_LIFE_WIDTH
             call cpct_drawSpriteBlended_asm
            ;///
        ;;//////////////////

        ;; ACTUALIZA LA VIDA
             ld a, (current_life_x)
             add #(SYS_SCORE_LIFE_WIDTH + SYS_SCORE_LIFE_X_MARGIN)
             ld (current_life_x), a
         ;///////////////////
         
        ret
;////////////////

;SYS_RENDER_DRAW_LIFES
    ;Dibuja la cantidad maxima de vidas al inicio de la partida
    sys_render_draw_lifes::        
        life_draw_loop:
         call sys_render_draw_one_life   

         ;; if(lifes != 0){ seguimos pintando
             ld a, (lifes)
             dec a
             ld (lifes), a
             or a
             jr nz, life_draw_loop
         ;; else { reseteamos el contador para empezar el juego
             ld a, (max_lifes)
             ld (lifes), a
         ;; }
    ret
;/////////////////

;SYS_RENDER_DELETE_ONE_LIFE
    ;Borra la vida mas a la derecha
    sys_render_delete_one_life::
        ;; Ajustamos la posicion de la vida a borrar
            ld a, (current_life_x)
            sub #(SYS_SCORE_LIFE_WIDTH + SYS_SCORE_LIFE_X_MARGIN)
            ld (current_life_x), a
        ;; ///////////////////////

        ;; BORRAMOS LA VIDA    
            ;; Memoria de video 
             ld de, #0xC000
             ld c, a
             ld b, #SYS_SCORE_LIFE_Y
             call cpct_getScreenPtr_asm
 
             ld de, #_sp_vida
             ex de, hl
             ld c, #SYS_SCORE_LIFE_HEIGHT
             ld b, #SYS_SCORE_LIFE_WIDTH
             call cpct_drawSpriteBlended_asm
            ;///

            ;; Doble buffer    
             ld de, #0x8000
             ld a, (current_life_x)
             ld c, a
             ld b, #SYS_SCORE_LIFE_Y
             call cpct_getScreenPtr_asm
 
             ld de, #_sp_vida
             ex de, hl
             ld c, #SYS_SCORE_LIFE_HEIGHT
             ld b, #SYS_SCORE_LIFE_WIDTH
             call cpct_drawSpriteBlended_asm
            ;;////
        ;;/////////////////////

        ret
;/////////////////

;DRAWCAJABOSS
    ;Dibuja una la caja del boss
    ;Destrois: AF, HL, BC, DE
    drawCajaBoss:
        ;Obtenemos la posicion de la primera mitad del suelo
        ld de, #first_buffer
        ld c, #CAJA_X
        ld b, #CAJA_Y
        push bc
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #dark_blue_color
        ld c, #CAJA_W
        ld b, #CAJA_H
        call cpct_drawSolidBox_asm

        
        pop bc
        ;Dibujamos las franjas negras 
        ld a, b
        add #FRANJA_Y_DISPLACEMENT
        ld b, a
        ld a, c
        add #FRANJA_X_DISPLACEMENT
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra
        
        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        call drawFranjaNegra

        ;Obtenemos la posicion del ojo izquierdo del boss
        ld de, #first_buffer
        ld c, #EYE_LEFT_X
        ld b, #EYE_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos el ojo izquierdo del boss
        ld de, #_sp_left_eye
        ex de, hl
        ld c, #SP_LEFT_EYE_W
        ld b, #SP_LEFT_EYE_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion del ojo derecho del boss
        ld de, #first_buffer
        ld c, #EYE_RIGHT_X
        ld b, #EYE_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos el ojo derecho del boss
        ld de, #_sp_right_eye
        ex de, hl
        ld c, #SP_RIGHT_EYE_W
        ld b, #SP_RIGHT_EYE_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la boca del boss
        ld de, #first_buffer
        ld c, #MOUTH_X
        ld b, #MOUTH_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la boca del boss
        ld de, #_sp_mouth
        ex de, hl
        ld c, #SP_MOUTH_W
        ld b, #SP_MOUTH_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la mano izquierda
        ld de, #first_buffer
        ld c, #HAND_LEFT_X
        ld b, #HAND_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la mano izquierda
        ld de, #_sp_hand_left
        ex de, hl
        ld c, #SP_HAND_LEFT_W
        ld b, #SP_HAND_LEFT_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la mano derecha
        ld de, #first_buffer
        ld c, #HAND_RIGHT_X
        ld b, #HAND_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la mano izquierda
        ld de, #_sp_hand_right
        ex de, hl
        ld c, #SP_HAND_RIGHT_W
        ld b, #SP_HAND_RIGHT_H
        call cpct_drawSprite_asm

        ld c, #CHAIN_LEFT_X
        ld b, #CHAIN_Y
        call drawChain

        ld c, #CHAIN_RIGHT_X
        ld b, #CHAIN_Y
        call drawChain
    ret
;/////////////////

;DRAWSADCAJABOSS
    ;Dibuja una la caja del boss
    ;Destrois: AF, HL, BC, DE
    drawSadCajaBoss:
        ;Obtenemos la posicion de la primera mitad del suelo
        ld de, #first_buffer
        ld c, #CAJA_X
        ld b, #CAJA_Y
        push bc
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #dark_blue_color
        ld c, #CAJA_W
        ld b, #CAJA_H
        call cpct_drawSolidBox_asm

        
        pop bc
        ;Dibujamos las franjas negras 
        ld a, b
        add #FRANJA_Y_DISPLACEMENT
        ld b, a
        ld a, c
        add #FRANJA_X_DISPLACEMENT
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra
        
        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        push bc
        call drawFranjaNegra

        pop bc
        ld a, c
        add #(FRANJA_X_DISPLACEMENT+FRANJA_W)
        ld c, a
        call drawFranjaNegra

        ;Obtenemos la posicion del ojo izquierdo del boss
        ld de, #first_buffer
        ld c, #EYE_LEFT_X
        ld b, #EYE_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos el ojo izquierdo del boss
        ld de, #_sp_left_eye_sad
        ex de, hl
        ld c, #SP_LEFT_EYE_W
        ld b, #SP_LEFT_EYE_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion del ojo derecho del boss
        ld de, #first_buffer
        ld c, #EYE_RIGHT_X
        ld b, #EYE_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos el ojo derecho del boss
        ld de, #_sp_right_eye_sad
        ex de, hl
        ld c, #SP_RIGHT_EYE_W
        ld b, #SP_RIGHT_EYE_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la boca del boss
        ld de, #first_buffer
        ld c, #MOUTH_X
        ld b, #MOUTH_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la boca del boss
        ld de, #_sp_mouth_sad
        ex de, hl
        ld c, #SP_MOUTH_W
        ld b, #SP_MOUTH_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la mano izquierda
        ld de, #first_buffer
        ld c, #HAND_LEFT_X
        ld b, #HAND_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la mano izquierda
        ld de, #_sp_hand_left
        ex de, hl
        ld c, #SP_HAND_LEFT_W
        ld b, #SP_HAND_LEFT_H
        call cpct_drawSprite_asm

        ;Obtenemos la posicion de la mano derecha
        ld de, #first_buffer
        ld c, #HAND_RIGHT_X
        ld b, #HAND_Y
        call cpct_getScreenPtr_asm

        ;Dibujamos la mano izquierda
        ld de, #_sp_hand_right
        ex de, hl
        ld c, #SP_HAND_RIGHT_W
        ld b, #SP_HAND_RIGHT_H
        call cpct_drawSprite_asm
    ret
;/////////////////

;DRAWFRANJANEGRA
    ;Dibuja una franja negra
    ;Inputs: C -> posX, B -> posY
    ;Destrois: AF, HL, BC, DE
    drawFranjaNegra:
        ;Obtenemos la posicion de la primera mitad del suelo
        ld de, #first_buffer
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #black_color
        ld c, #FRANJA_W
        ld b, #FRANJA_H
        call cpct_drawSolidBox_asm
        
    ret
;/////////////////

;DRAWCHAIN
    ;Dibuja una cadena
    ;Inputs: C -> posX, B -> posY
    ;Destrois: AF, HL, BC, DE
    drawChain:
        push bc

        ;Obtenemos la posicion de una cadena
        ld de, #first_buffer
        call cpct_getScreenPtr_asm

        ;Dibujamos una cadena
        ex de, hl
        ld hl, #_sp_chain
        ld c, #SP_CHAIN_W
        ld b, #SP_CHAIN_H
        call cpct_drawSprite_asm

        pop bc

        ;Obtenemos la posicion de una cadena rotada
        ld de, #first_buffer
        ld a, b
        add #SP_CHAIN_H
        ld b, a
        push bc
        ld a, c
        add #1
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos la primera mitad del suelo (No se puede hacer drawSolidBox de mas de 64 bytes)
        ex de, hl
        ld a, #rotated_chain_color
        ld c, #ROTATED_CHAIN_W
        ld b, #ROTATED_CHAIN_H
        call cpct_drawSolidBox_asm

        pop bc
        ;Obtenemos la posicion de una cadena
        ld de, #first_buffer
        ld a, b
        add #ROTATED_CHAIN_H
        ld b, a
        push bc
        call cpct_getScreenPtr_asm

        ;Dibujamos una cadena
        ex de, hl
        ld hl, #_sp_chain
        ld c, #SP_CHAIN_W
        ld b, #SP_CHAIN_H
        call cpct_drawSprite_asm

        pop bc

        ;Obtenemos la posicion de una cadena rotada
        ld de, #first_buffer
        ld a, b
        add #SP_CHAIN_H
        ld b, a
        push bc
        ld a, c
        add #1
        ld c, a
        call cpct_getScreenPtr_asm

        ;Dibujamos una cadena rotada
        ex de, hl
        ld a, #rotated_chain_color
        ld c, #ROTATED_CHAIN_W
        ld b, #ROTATED_CHAIN_H
        call cpct_drawSolidBox_asm

        pop bc
        ;Obtenemos la posicion de una cadena
        ld de, #first_buffer
        ld a, b
        add #ROTATED_CHAIN_H
        ld b, a
        call cpct_getScreenPtr_asm

        ;Dibujamos una cadena
        ex de, hl
        ld hl, #_sp_chain
        ld c, #SP_CHAIN_W
        ld b, #SP_CHAIN_H
        call cpct_drawSprite_asm
    ret
;/////////////////

;DRAWPUERTAS
    drawPuertas:
        ;Obtenemos la posicion de la puerta izquierda
        ld de, (current_screen_to_draw)
        ld c, #PUERTA_LEFT_X
        ld b, #PUERTA_Y
        call cpct_getScreenPtr_asm
        
        ;Dibujamos la puerta izquierda
        ex de, hl
        ld a, #color_red
        ld c, #PUERTA_W
        ld b, #PUERTA_H
        call cpct_drawSolidBox_asm

        ;Obtenemos la posicion de la puerta derecha
        ld de, (current_screen_to_draw)
        ld c, #PUERTA_RIGHT_X
        ld b, #PUERTA_Y
        call cpct_getScreenPtr_asm
        
        ;Dibujamos la puerta derecha
        ex de, hl
        ld a, #color_red
        ld c, #PUERTA_W
        ld b, #PUERTA_H
        call cpct_drawSolidBox_asm


        call drawPomos
    ret
;/////////////////

;DRAWPOMOS
    drawPomos:
        ;Obtenemos la posicion del pomo izquierdo
        ld de, (current_screen_to_draw)
        ld c, #POMO_LEFT_X
        ld b, #POMO_Y
        call cpct_getScreenPtr_asm
        
        ;Dibujamos el pomo izquierdo
        ex de, hl
        ld a, #00
        ld c, #POMO_W
        ld b, #POMO_H
        call cpct_drawSolidBox_asm

        ;Obtenemos la posicion del pomo derecho
        ld de, (current_screen_to_draw)
        ld c, #POMO_RIGHT_X
        ld b, #POMO_Y
        call cpct_getScreenPtr_asm
        
        ;Dibujamos del derecho
        ex de, hl
        ld a, #00
        ld c, #POMO_W
        ld b, #POMO_H
        call cpct_drawSolidBox_asm
    ret
;/////////////////

;DRAWSCORELINE
    ;Inputs: IY -> Posicion de memoria del label, HL -> Score a pintar, C -> PosX, B -> PosY 
    drawScoreLine:
        push hl
        push bc 

        call drawString

        pop bc
        pop hl 
        push hl
  
        ld a, c
        add #(SCORE_LABEL_LENGHT)
        ld c, a
        push bc
        call drawByte_C000

        pop bc
        pop hl
        ;Pintamos el byte menor del score
        ld h, l  
        ld a,c
        add #(CHARACTER_BYTES_W*2) ;Hay que dibujar en la misma posicion que antes, dos caracteres mas a la derecha
        ld c, a
        call drawByte_C000
    ret
;/////////////////

;DRAWSTRING
    ;Dibuja un string
    ;Inputs: IY -> Posicion de memoria del string, C -> PosX, B -> PosY
    drawString:
        ld de, #0xC000
        call cpct_getScreenPtr_asm

        call cpct_drawStringM0_asm
    ret
;/////////////////

;DRAWSCORESUMMARY
    drawScoreSummary:
        ld iy, #score_label
        ld hl, (score)
        ld c, #SCORE_LABEL_X
        ld b, #SCORE_LABEL_Y
        call drawScoreLine

        ld iy, #score_total_label
        ld hl, (score_total)
        ld c, #SCORE_TOTAL_LABEL_X
        ld b, #SCORE_TOTAL_LABEL_Y
        call drawScoreLine
    ret
;////////////////

;SYS_RENDER_CLEANSCREENBLACK
    sys_render_cleanScreenBlack:
        ld de, #second_buffer       ;; /
        ld a, #black_color          ;; | Pintamos los 2 buffers de negro
        ld bc, #0x8000              ;; |
        call cpct_memset_asm        ;; \
    ret
;////////////////

;DRAWENEMIESLEFTLABEL
    ;Dibuja el label de enemigos restantes
    drawEnemiesLeftLabel:
        ;Dibujado del sprite
        ld de, #first_buffer
        ld c, #ENTITIES_LEFT_LABEL_X
        ld b, #ENTITIES_LEFT_LABEL_Y
        call cpct_getScreenPtr_asm

        
        ld de, #_sp_enemies
        ex de, hl
        ld c, #SP_ENEMIES_W
        ld b, #SP_ENEMIES_H
        call cpct_drawSprite_asm

        ;Dibujado de la 'X'
        ld iy, #entities_left_label
        ld c, #ENTITIES_LEFT_X_X
        ld b, #ENTITIES_LEFT_X_Y
        call drawString
    ret
;////////////////
