;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;         Copyright (C) 2022,  Panetonent                                             ;;;
;;;                                                                                     ;;;
;;;         This program is free software: you can redistribute it and/or modify        ;;;
;;;         it under the terms of the GNU General Public License as published by        ;;;
;;;         the Free Software Foundation, either version 3 of the License, or           ;;;
;;;         (at your option) any later version.                                         ;;;
;;;                                                                                     ;;;
;;;         This program is distributed in the hope that it will be useful,             ;;;
;;;         but WITHOUT ANY WARRANTY; without even the implied warranty of              ;;;
;;;         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ;;;
;;;         GNU General Public License for more details.                                ;;;
;;;                                                                                     ;;;
;;;         You should have received a copy of the GNU General Public License           ;;;
;;;         along with this program.  If not, see <http://www.gnu.org/licenses/>        ;;;
;;;                                                                                     ;;;
;;;                                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.include "cpctelera.h.s"
.include "math.h.s"

;Memoria
    chunk_width:
        .db 1
    chunk_count:
        .db #TOOLS_TRIANGLE_CHUNK_COUNT
    chunk_x:
        .db 40
    chunk_y:
        .db 0
;//////////





;TOOLS_MATH_BCDADD
    ;Suma dos valores y devuelve el resultado en decimal gracias al uso del BCD (Binary coded decimal)
    ;Inputs:    HL -> Sumando decimal, BC -> El otro sumando
    ;Outputs:   HL -> Resultado en decimal
    ;Destrois:  AF
    tools_math_bcdadd:
        ld a, l     ;/     
        add c       ;|Le sumamos los dos primeros digitos de bc a hl
        daa         ;|y lo formateamos en decimal (bcd)
        ld l, a     ;\
        
        ld a, h     ;/
        adc b       ;|Sumamos los dos digitos mayores teniendo en cuenta el acarreo    
        daa         ;|de la suma anterior    
        ld h, a     ;\
    ret
;/////////////////

;TOOLS_MATH_BCDSUB
    ;Resta dos valores y devuelve el resultado en decimal gracias al uso del BCD (Binary coded decimal)
    ;Inputs:    HL -> Sumando decimal, BC -> El otro sumando
    ;Outputs:   HL -> Resultado en decimal
    ;Destrois:  AF
    tools_math_bcdsub:
        ld a, l     ;/     
        sub c       ;|Le restamos los dos primeros digitos de bc a hl
        daa         ;|y lo formateamos en decimal (bcd)
        ld l, a     ;\
        
        ld a, h     ;/
        sbc b       ;|Restamos los dos digitos mayores teniendo en cuenta el acarreo    
        daa         ;|de la resta anterior    
        ld h, a     ;\
    ret
;/////////////////


;TOOLS_MATH_DRAWTRIANGLE
    ;Dibuja un triangulo    
    tools_math_drawtriangle::
        ld a, #TOOLS_TRIANGLE_INIT_X
        ld (chunk_x), a
        ld a, #TOOLS_TRIANGLE_INIT_Y
        ld (chunk_y), a
        ld a, #TOOLS_TRIANGLE_INIT_WIDTH
        ld (chunk_width), a
        ld a, #TOOLS_TRIANGLE_CHUNK_COUNT
        ld (chunk_count), a
        triangle_loop:
            ;; PINTAMOS
                ld de, #0xC000
                ld hl, #chunk_x
                ld c, (hl)
                ld hl, #chunk_y
                ld b, (hl)

                call cpct_getScreenPtr_asm

                ex de, hl
                ld a, #00
                ld hl, #chunk_width
                ld c, (hl)
                ld b, #TOOLS_TRIANGLE_CHUNK_HEIGHT

                call cpct_drawSolidBox_asm

                ld de, #0x8000
                ld hl, #chunk_x
                ld c, (hl)
                ld hl, #chunk_y
                ld b, (hl)

                call cpct_getScreenPtr_asm

                ex de, hl
                ld a, #00
                ld hl, #chunk_width
                ld c, (hl)
                ld b, #TOOLS_TRIANGLE_CHUNK_HEIGHT

                call cpct_drawSolidBox_asm
            ;; //////////////
            
            ; if(contador == 0) salimos del metodo
            ld a, (chunk_count)
            dec a
            ld (chunk_count), a
            or a
            jr z, end_triangle_loop

            ; if(contador - start_reduce == 0) empezamos la reduccion
            cp #TOOLS_TRIANGLE_START_REDUCE
            jr c, reducing_triangle_loop
                ;; ACTUALIZAMOS VALORES
                    ;; Actualizamos el valor de X para la siguiente iteracion
                    ld a, (chunk_x)
                    dec a
                    ld (chunk_x), a

                    ;; Actualizamos el valor de Y para la siguiente iteracion
                    ld a, (chunk_y)
                    add #TOOLS_TRIANGLE_CHUNK_HEIGHT
                    ld (chunk_y), a

                    ;; Actualizamos el valor del width para la siguiente iteracion
                    ld a, (chunk_width)
                    add #0x02
                    ld (chunk_width), a
                ;;//////////////////
                jr triangle_loop
            reducing_triangle_loop:
                ;; ACTUALIZAMOS VALORES
                        ;; Actualizamos el valor de X para la siguiente iteracion
                        ld a, (chunk_x)
                        add #03
                        ld (chunk_x), a

                        ;; Actualizamos el valor de Y para la siguiente iteracion
                        ld a, (chunk_y)
                        add #TOOLS_TRIANGLE_CHUNK_HEIGHT_REDUCING
                        ld (chunk_y), a

                        ;; Actualizamos el valor del width para la siguiente iteracion
                        ld a, (chunk_width)
                        sub #0x06
                        ld (chunk_width), a
                    ;;//////////////////
                jr triangle_loop
        end_triangle_loop:

    ret
;/////////////////